from datetime import datetime
from uuid import UUID

from pydantic import ConfigDict
from backend.schemas.main_model import MainModel

class HouseBase(MainModel):
  icon: str
  name: str
  address: str

class AreaBase(MainModel):
  name: str
  desc: str

class AreaUpdate(AreaBase):
  id: UUID | None = None
  model_config = ConfigDict(from_attributes=True)

class HouseCreate(HouseBase):
  areas: list[AreaBase]
  model_config = ConfigDict(from_attributes=True)

class HouseUpdate(HouseBase):
  id: UUID
  areas: list[AreaUpdate]
  model_config = ConfigDict(from_attributes=True)

class HouseResponse(HouseBase):
  id: UUID
  areas: list[AreaUpdate]
  model_config = ConfigDict(from_attributes=True)

class HousesList(HouseBase):
  id: UUID
  created_at: datetime
  updated_at: datetime
  deleted_at: datetime | None

class HousesListResponse(MainModel):
  total: int
  list: list[HousesList]
